#include "local.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "hardlock.h"
#include "hl.h"

int hlport;
static int debug = FALSE;
#define DEBUG		if ( debug ) printf

LOCAL_FN void put_time();
LOCAL_FN void tohighlow();
LOCAL_FN void disable_hl();
LOCAL_FN void get_time();

char *
gethostid()
{
	static char host[HL_HOSTID_LEN*2+1];

	int registr,data,status,blkcnt;
	int hladdress;
	int set_clock_back;
	int orig_set_clock_back;
	long second_in_day;
	int access_count;
	int access_count2;
	char tmpdatastr[129];
	char customer[HL_CUSTOMER_LEN*2+1];
	int i;
	int lobyte, hibyte;
	time_t timeofday;
	time_t origtime, updatetime, endtime, setbacktime;
	int serial_number;
	int clock_set_back;

	/* Initialize parameters */
	hlport = 0;
	status=0;
	blkcnt=1;
	strcpy(tmpdatastr,"12345678");
	hladdress = HL_ADDRESS;

	/* Low level routines require that you pass the address of Parallel
	** port where Hardlock is located.  You may use HLCRYPT routine to
	** return the address of the port.
	*/
	hlcrypt (&status,&hlport,&hladdress,tmpdatastr,&blkcnt);
	if (status != 0 ) {
		fprintf(stderr,"No valid hardware key found, Status = %d\n",
			status);
		return(NULL);
	}

	/* Turn off the interrupts */
	intoff();
	hlon(&hlport, &hladdress);

	/* Turn on hardlock */
	registr= HL_LOC_VERSION;
	hlrwd(&hlport,&registr,&data);
	DEBUG("Version: %d\n",data);
	if ( HL_VERSION_NUMBER != data ) {
		fprintf(stderr,"Invalid Hardware key version number: %d\n",data);
		return(NULL);
	}

	/* Read the hostid */
	for (i=0; i<HL_HOSTID_LEN; ++i) {
		registr = HL_LOC_HOSTID+i;
		hlrwd(&hlport,&registr,&data);
		tohighlow(data,&lobyte,&hibyte);
		host[2*i] = hibyte;
		host[2*i+1] = lobyte;
	}
	host[HL_HOSTID_LEN*2] = '\0';
	DEBUG("Host: %s\n",host);

	/* Read the customer */
	for (i=0; i<HL_CUSTOMER_LEN; ++i) {
		registr = HL_LOC_CUSTOMER+i;
		hlrwd(&hlport,&registr,&data);
		tohighlow(data,&lobyte,&hibyte);
		customer[2*i] = hibyte;
		customer[2*i+1] = lobyte;
	}
	customer[HL_CUSTOMER_LEN*2] = '\0';
	DEBUG("Customer: %s\n",customer);

	/* Verify that the lock is enabled */
	registr= HL_LOC_ENABLE;
	hlrwd(&hlport,&registr,&data);
	DEBUG("Enable: %d\n",data);
	if ( data != HL_ENABLE ) {
		fprintf(stderr,"Key is no longer enabled.\n");
		disable_hl(KEY_NO_CHANGE);
		return(NULL);
	}

	get_time(&origtime, HL_LOC_ORIG_TIME);
	get_time(&updatetime, HL_LOC_UPDATE_TIME);
	get_time(&setbacktime, HL_LOC_SET_BACK_TIME);
	get_time(&endtime, HL_LOC_END_TIME);

	/* Get SET_CLOCK_BACK */
	registr= HL_LOC_SET_CLOCK_BACK;
	hlrwd(&hlport,&registr,&set_clock_back);
	orig_set_clock_back = set_clock_back;

	timeofday = 0;
	timeofday = time( &timeofday);
	DEBUG("Present time: %ld\n",timeofday);

	/* Serial Number */
	registr= HL_LOC_SN;
	hlrwd(&hlport,&registr,&serial_number);
	DEBUG("Serial number: %d\n",serial_number);

	/* Get access count and increment it by 1 */
	registr= HL_LOC_ACCESS_COUNT;
	hlrwd(&hlport,&registr,&access_count);
	DEBUG("Serial number: %d\n",access_count);
	if ( access_count < 0 || access_count > 10000 ) {
		registr= HL_LOC_ACCESS_COUNT2;
		hlrwd(&hlport,&registr,&access_count2);
		++access_count2;
		hlwwd(&hlport,&registr,&access_count2);
	} else {
		++access_count;
		hlwwd(&hlport,&registr,&access_count);
	}

	/* Update access count */

	/* Check times -- if bad disable key */
	
	if ( timeofday <= origtime ) {
		fprintf(stderr,"Clock on your PC is set incorrectly\n");
		DEBUG("Time difference: %ld\n",timeofday-origtime);
		return(NULL);
	}

	if ( timeofday <= setbacktime ) {
		fprintf(stderr,"Key is no longer enabled.\n");
		fprintf(stderr,"To reset your hardware key call your Sonnet Software.\n");
		fprintf(stderr,"representative.\n");
		disable_hl(KEY_CLOCK_RESET_TWICE);
		return(NULL);
	}

	clock_set_back = FALSE;
	if ( timeofday <= updatetime ) {
		/* Clock has been set back */
		second_in_day = (long)24*60*60;
		if ( timeofday + 2*second_in_day < updatetime ) {
			fprintf(stderr,"Clock on your PC was set back\n");
			return(NULL);
		}
		clock_set_back = TRUE;
	}

	/* Check if new time is earlier than last end time.  If so	*/
	/* Assume that the clock has been set back 			*/
	/* Note: the user may have ran a sonnet program then but the	*/
	/* machine crashed and it never finish.  			*/
	if ( endtime > updatetime && timeofday <= endtime ) {
		clock_set_back = TRUE;
	}

	if ( clock_set_back ) {
		++set_clock_back;
		if ( set_clock_back > 4 ) {
			fprintf(stderr,"Key is no longer enabled.\n");
			fprintf(stderr,"To reset your hardware key call your Sonnet Software.\n");
			fprintf(stderr,"representative.\n");
			disable_hl(KEY_CONT_SET_BACK);
			return(NULL);
		}

		/* Clock was set back -- update the last set back time */
		put_time(HL_LOC_SET_BACK_TIME,timeofday);
	}

	
	/* Write present time out to module */
	put_time(HL_LOC_UPDATE_TIME,timeofday);
	
	/* Write # times clock has been set back */
	if ( orig_set_clock_back != set_clock_back ) {
		registr= HL_LOC_SET_CLOCK_BACK;
		hlwwd(&hlport,&registr,&set_clock_back);
	}

	/* Turn on the interrupts */
	hloff(&hlport);
	inton();

	return (host);
}

LOCAL_FN void
tohighlow(data,lobyte,hibyte)
int data;
int *lobyte;
int *hibyte;
{
	*hibyte = data /256;
	*lobyte = data % 256;
}

LOCAL_FN void
disable_hl(rc)
int rc;
{
	int registr, data;

	if ( rc != KEY_NO_CHANGE ) {
		registr= HL_LOC_DISABLE_REASON;
		data = rc;
		hlwwd(&hlport,&registr,&data);
	}
	registr= HL_LOC_ENABLE;
	data = HL_DISABLE;
	hlwwd(&hlport,&registr,&data);
	hloff(&hlport);
	inton();
}

LOCAL_FN void
get_time(ptime, loc)
time_t *ptime; 
int loc;
{
	time_t time;
	int registr;
	long int long_time1, long_time2;
	int time1, time2;

	/* Get original time */
	registr= loc;
	hlrwd(&hlport,&registr,&time1);
	++registr;
	hlrwd(&hlport,&registr,&time2);
	long_time1 = time1;
	long_time2 = time2;
	time = long_time1<<(long)16 + long_time2;
	time = long_time1;
	time <<= 16;
	time += long_time2;
	DEBUG("Original time: %ld\n",time);
	*ptime = time;
}

verify_hardware_key()
{
	int registr,data,status,blkcnt;
	int check_sum;
	int i;
	int port;
	int hladdress;
	char tmpdatastr[9];


        /* Initialize parameters */
        port = 0;
        hladdress = HL_ADDRESS-1;
        status=0;
        blkcnt=1;
	hladdress += blkcnt;
        strcpy(tmpdatastr,"ABCDEFGH");

        /* 
	** Low level routines require that you pass the address of Parallel
        ** port where Hardlock is located.  You may use HLCRYPT routine to
        ** return the address of the port.
        */
        hlcrypt (&status,&port,&hladdress,tmpdatastr,&blkcnt);
        if (status != 0 ) {
                printf("No valid hardware key found, status = %d\n",status);
                exit_program(1);
        }

	intoff();
	hlon(&port,&hladdress);

	check_sum = 0;
	for (i=30; i<47; ++i ) {
		registr=i;
		hlrwd(&port,&registr,&data);
		check_sum ^= data;
	}
	registr = HL_LOC_CHECK_SUM;
        hlrwd(&port,&registr,&data);
        if ( check_sum != data ) {
                fprintf(stderr,"Hardware key check sum failure.\n");
                fprintf(stderr,"Report this problem to your Sonnet Software representative.\n");
		exit_program(1);
	}
	hloff(&port);
	inton();
}

update_key_end_time()
{
	int port;
	int hladdress = HL_ADDRESS;
	int tmpdatastr[9],status,blkcnt;
	time_t timeofday;


	port = 0;
	blkcnt = 1;
	strcpy(tmpdatastr,"11111111");
	hlcrypt (&status,&port,&hladdress,tmpdatastr,&blkcnt);
	if ( status == 0 ) {
		intoff();
		hlon(&port,&hladdress);

		/* Save end time */
		timeofday = 0;
		timeofday = time( &timeofday);

		put_time(HL_LOC_END_TIME,timeofday);

		hloff(&port);
		inton();
	}
}

LOCAL_FN void
put_time(loc,time)
int loc;
time_t time;
{
	int registr,data;
	time_t lotime;
	time_t hitime;

	lotime = hitime = time;
	lotime <<= 16;
	lotime >>= 16;
	hitime >>= 16;
	registr = loc;
	data = (int)hitime;
	hlwwd(&hlport,&registr,&data);
	++registr;
	data = (int)lotime;
	hlwwd(&hlport,&registr,&data);
}
